package br.com.ages.mutiraodobemback.service.event;

import br.com.ages.mutiraodobemback.domain.EventEntity;
import br.com.ages.mutiraodobemback.dto.EventRequestDTO;
import br.com.ages.mutiraodobemback.dto.EventResponseDTO;
import br.com.ages.mutiraodobemback.repository.IEventRepository;
import br.com.ages.mutiraodobemback.util.Validations;
import br.com.ages.mutiraodobemback.service.user.IUserService;
import br.com.ages.mutiraodobemback.util.exceptions.CustomIllegalArgumentException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.stream.Collectors;

import static br.com.ages.mutiraodobemback.message.Message.*;

@Service
public class EventServiceImpl implements IEventService {

    private final IEventRepository repository;
    private final IUserService userService;

    @Autowired
    public EventServiceImpl(IEventRepository repository, IUserService userService) {
        this.repository = repository;
        this.userService = userService;
    }

    @Override
    public EventResponseDTO save(EventRequestDTO event) {
        validateFields(event);
        EventEntity eventEntity = new EventEntity(event);
        eventEntity.setCreatedDate(LocalDateTime.now());
        eventEntity.setActive(true);
        eventEntity.setUser(userService.getLoggedUser());
        return new EventResponseDTO(repository.save(eventEntity));
    }

    @Override
    public EventResponseDTO update(EventRequestDTO event) {
        return this.save(event);
    }

    @Override
    public List<EventRequestDTO> findAll() {
        return repository.findAll().stream().map(EventRequestDTO::new).collect(Collectors.toList());
    }

    @Override
    public List<EventResponseDTO> findAllActive() {
        return repository.findAllByActiveIsTrue().stream().map(EventResponseDTO::new).collect(Collectors.toList());
    }


    private void validateFields(EventRequestDTO event) {
        if (Validations.isEmptyOrisBlank(event.getTitle()))
            throw new CustomIllegalArgumentException(NOME_OBRIGATORIO);

        if (Validations.isEmptyOrisBlank(event.getStreet()))
            throw new CustomIllegalArgumentException(ENDERECO_OBRIGATORIO);

        if (Validations.isEmptyOrisBlank(event.getStartDate().toString()))
            throw new CustomIllegalArgumentException(DATA_INICIAL_OBRIGATORIO);

        if (Validations.isEmptyOrisBlank(event.getEndDate().toString()))
            throw new CustomIllegalArgumentException(DATA_FINAL_OBRIGATORIO);

        if (Validations.isAfter(event.getStartDate(), event.getEndDate()))
            throw new CustomIllegalArgumentException(DATA_INICIAL_MAIOR_QUE_DATA_FINAL);

        if (Validations.isEmptyOrisBlank(event.getDescription()))
            throw new CustomIllegalArgumentException(DESCRICAO_OBRIGATORIO);

        if (Validations.isEmptyOrisBlank(event.getLatitude()))
            throw new CustomIllegalArgumentException(LATITUDE_OBRIGATORIO);

        if (Validations.isEmptyOrisBlank(event.getLongitude()))
            throw new CustomIllegalArgumentException(LONGITUDE_OBRIGATORIO);

        // TODO desobrir o que é EventType
//        if (event.getEventType().isBlank() || event.getEventType().isEmpty())
//            throw new CustomIllegalArgumentException(EVENTO_OBRIGATORIO);

    }
}
