package br.com.ages.mutiraodobemback.service.user;

import br.com.ages.mutiraodobemback.domain.UserEntity;
import br.com.ages.mutiraodobemback.dto.UserRequestDTO;
import br.com.ages.mutiraodobemback.dto.UserResponseDTO;
import br.com.ages.mutiraodobemback.repository.IUserRepository;
import br.com.ages.mutiraodobemback.util.MD5PasswordEncoder;
import br.com.ages.mutiraodobemback.util.exceptions.CustomIllegalArgumentException;
import br.com.ages.mutiraodobemback.util.exceptions.UserAlreadyRegistered;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import static br.com.ages.mutiraodobemback.message.Message.*;

@Service
public class UserServiceImpl implements IUserService {

    private final IUserRepository repository;

    @Autowired
    public UserServiceImpl(IUserRepository repository) {
        this.repository = repository;
    }

    @Override
    public UserResponseDTO save(UserRequestDTO user) {
        this.validateFields(user);

        String encryptedPassword = new MD5PasswordEncoder().encode(user.getPassword());
        UserEntity newUser = UserEntity.builder()
                .name(user.getName())
                .document(user.getDocument())
                .type(user.getType())
                .phone(user.getPhone())
                .email(user.getEmail())
                .password(encryptedPassword)
                .image(user.getImage())
                .role("USER")
                .active(true)
                .build();
        UserEntity userSaved = this.repository.save(newUser);
        return UserResponseDTO.builder()
                .id(userSaved.getId())
                .name(userSaved.getName())
                .document(userSaved.getDocument())
                .email(userSaved.getEmail())
                .phone(userSaved.getPhone())
                .image(userSaved.getImage())
                .build();
    }

    @Override
    public List<UserResponseDTO> findAll() {
        return this.repository.findAll().stream().map(u -> UserResponseDTO.builder()
                .id(u.getId())
                .name(u.getName())
                .document(u.getDocument())
                .email(u.getEmail())
                .build()).collect(Collectors.toList());
    }

    @Override
    public UserEntity getLoggedUser() {
        Authentication auth = SecurityContextHolder.getContext().getAuthentication();
        String email = auth.getPrincipal().toString();
        return this.repository.findByEmail(email).orElse(null);
    }

    private void validateFields(UserRequestDTO user) {
        if (user.getName().isBlank() || user.getName().isEmpty())
            throw new CustomIllegalArgumentException(NOME_OBRIGATORIO);

        if (user.getDocument().isBlank() || user.getDocument().isEmpty())
            throw new CustomIllegalArgumentException(CPF_CNPJ_OBRIGATORIO);

        if (user.getPassword().isBlank() || user.getPassword().isEmpty())
            throw new CustomIllegalArgumentException(SENHA_OBRIGATORIO);

        if (user.getPhone().isBlank() || user.getPhone().isEmpty())
            throw new CustomIllegalArgumentException(TELEFONE_OBRIGATORIO);

        if (user.getEmail().isBlank() || user.getEmail().isEmpty())
            throw new CustomIllegalArgumentException(EMAIL_OBRIGATORIO);

        if (user.getImage().isBlank() || user.getImage().isEmpty())
            throw new CustomIllegalArgumentException(IMAGEM_OBRIGATORIO);

        Optional<UserEntity> userRegisteredWithEmail = this.repository.findByEmail(user.getEmail());
        if(userRegisteredWithEmail.isPresent())
            throw new UserAlreadyRegistered(EMAIL_EXISTENTE);

        Optional<UserEntity> userRegisteredWithDocument = this.repository.findByDocument(user.getDocument());
        if(userRegisteredWithDocument.isPresent())
            throw new UserAlreadyRegistered(CPF_CNPJ_EXISTENTE);
    }

}
